package org.example;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Random;
import java.util.concurrent.CompletableFuture;

import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.s3.S3AsyncClient;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;
import software.amazon.awssdk.services.s3.model.GetObjectResponse;
import software.amazon.awssdk.services.s3.model.PutObjectRequest;

public class Main
{
    public static void main(String[] args) throws IOException, InterruptedException
    {
        String ACCESS_KEY = args[0];
        String SECRET_KEY = args[1];
        String ENDPOINT = args[2];
        String REGION = args[3];
        String BUCKET = args[4];

        S3AsyncClient client = S3AsyncClient.builder()
                .forcePathStyle(true)
                .credentialsProvider(StaticCredentialsProvider.create(AwsBasicCredentials.create(ACCESS_KEY, SECRET_KEY)))
                .endpointOverride(URI.create(ENDPOINT))
                .region(Region.of(REGION)).build();

        // Create a 1GB file for testing
        byte[] bytes = new byte[1024 * 1024 * 1024];
        new Random(894843892).nextBytes(bytes);
        Path uploadPath = Path.of("/tmp/upload/1gb.bin");
        Files.createDirectories(uploadPath.getParent());
        Files.write(uploadPath, bytes);
        String key = "1gb.bin";
        client.putObject(PutObjectRequest.builder().bucket(BUCKET).key(key).build(), uploadPath).join();
        System.out.println("File with 1GB contentLength upload successfully: " + BUCKET + "/" + key);

        Path downloadPath = Path.of("/tmp/download/1gb.bin");
        Files.createDirectories(downloadPath.getParent());
        System.out.println("Starting first download attempt: " + BUCKET + "/" + key + " to " + downloadPath);
        CompletableFuture<GetObjectResponse> future = client.getObject(GetObjectRequest.builder().bucket(BUCKET).key(key).build(), downloadPath);
        System.out.println("enforcing a delete after 5 seconds from starting the download");
        Thread.sleep(5_000L);
        if (future.isDone())
        {
            throw new RuntimeException("Future completed before 5 seconds, trying using a shorter sleep time (repro failed)");
        }
        Files.deleteIfExists(downloadPath);
        future.whenComplete((response, exception) -> {
            if (exception != null)
                System.out.println("File with 1GB contentLength download failed: " + exception.getMessage());
            else if (!Files.exists(downloadPath))
                System.out.println("Future completed with no errors, but file not found at: " + downloadPath);
            else
                System.out.println("File with 1GB contentLength download successfully: " + downloadPath + " (repro failed)");
        }).join();
    }
}